import { NS, AutocompleteData } from '@ns'
import { parse } from './lib/flag';
import { sprintf } from './lib/printf';
import {calculateBatchResource} from './lib/batchbase';

// eslint-disable-next-line require-await
export async function main(ns: NS) : Promise<void> {
    const flag = parse(ns.args.map(x=>x.toString()));
    if(flag.h || flag.help){
        const msg = ['run cmd [startThread] [--hostname || -h] [--graph]']
        msg.forEach(x=>ns.tprint(x));
        return;
    }
    if(flag._.length === 0){
        ns.tprint("Error. Require Argument");
        return;
    }
    if(!(flag.host)){
        ns.tprint("host required");
        return;
    }
    const v = parseInt(flag._[0].toString());
    const hostname = flag.host ?? flag.t;
    const server = ns.getServer(hostname)
    if(flag.graph){
        const col = [];
        for (let i = 1; i < v; i++) {
            const info = calculateBatchResource(ns,hostname,i,server.moneyMax,1);
            col.push({
                costEffect :(info.earnMoneyPerSec / info.totalThreadCount),
                index: i,
                total: info.totalThreadCount
            })
        }
        col.forEach(({costEffect,index,total})=>{
            ns.tprint(sprintf("%3d %10.2f %4d",index,costEffect,total));
        })
        ns.tprint(col.map(x=>x.costEffect))
        return;
    }
    const info = calculateBatchResource(ns,hostname,v,server.moneyMax,1);
    ns.tprint(JSON.stringify(info,undefined,2));
    return;
}

// eslint-disable-next-line @typescript-eslint/no-unused-vars
export function autocomplete(data : AutocompleteData, args : string[]) : string[] {
    return [...data.servers]
}