import {NS} from "@ns"
import { parse } from "./lib/flag";
import {getHackability,getLockSymbol} from "./lib/hack";
import { sprintf } from "./lib/printf";


let detail = false;

/** @param {NS} ns */
// eslint-disable-next-line require-await
export async function main(ns: NS): Promise<void> {
    const flag = parse(ns.args.map(String));
    detail = flag.d || flag.detail || false;
    
    const a = findServer(ns, 'home', 'home', 1);
    ns.tprint("\n"+a.join("\n"));
}

/** @param {NS} ns
 * @param {string} startServer
 * @param {string} targetServer
 * @param {number} depth
*/
function findServer(ns: NS, startServer: string, targetServer: string, depth: number): string[] {
    const servers = ns.scan(targetServer)
        .filter((server) => server !== startServer);
    return servers.map((server, i, arr) => {
        const lock = getHackability(ns, server);
        const lock_symbol = getLockSymbol(lock);
        const info = ns.getServer(server);
        const backdoorSymbol = info.backdoorInstalled ? '✅': '❌';
        let ret: string[] = [];

        const startSymbol = (i + 1 == arr.length) ? "└" : "├";
        const extendSymbol = (i + 1 == arr.length) ? " " : "│";
        //printf(ns, `😹${'>'.repeat(depth)} %s %s %s`,lock_symbol,server,info.backdoorInstalled ? '✅': '❌')
        const fmsg = (`${startSymbol}${lock_symbol} ${backdoorSymbol} ${server}`);
        ret.push(fmsg);
        if(detail){
            const moneyAvailable = ns.nFormat(info.moneyAvailable,"$0.000a");
            const moneyMax = ns.nFormat(info.moneyMax,"$0.000a");
            ret.push(sprintf("%s └%s🛡️ %6.2f/%6.2f(%3d),💸 %10s/%10s 💾%d/%d",extendSymbol,
                "-".repeat(20-depth),
                info.hackDifficulty,info.minDifficulty,info.requiredHackingSkill,
                moneyAvailable,moneyMax,
                info.maxRam, info.ramUsed));
        }
        if (lock !== "impossible") {
            const s = findServer(ns, targetServer, server, depth + 1).map(x=>`${extendSymbol}${x}`);
            ret = ret.concat(s)
        }
        return ret;
    }).flat();
}
