import { NS, AutocompleteData } from '@ns'
import {selectAllContract, getContractList, RemoteFilePath} from "lib/contract";
import { sprintf } from './lib/printf';
import { parse } from './lib/flag';
import Fuse from "lib/fuse.esm";

function searchFilename(ns: NS, filename: string, hostname?: string): RemoteFilePath | null{
    let p: RemoteFilePath[];
    if(hostname){
        p = getContractList(ns,hostname);
    }
    else{
        p = selectAllContract(ns);
    }
    const fuse = new Fuse(p, {includeScore:true, keys:["filename"]});
    const candiates = fuse.search(filename);
    if(candiates.length === 0){
        return null;
    }
    const candiate = candiates[0];
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    if(candiate!.score! > 0.8){
        return null;
    }
    return candiate.item;
}

// eslint-disable-next-line require-await
export async function main(ns : NS) : Promise<void> {
    const capi = ns.codingcontract;
    if(ns.args.length === 0){
        const ctList = selectAllContract(ns);
        ctList.map(ct=>{
            const type = capi.getContractType(ct.filename,ct.hostname);
            return {
                ...ct,
                type
            };
        }).sort((a,b)=>(a.type>b.type) ? 1 : -1)
        .forEach(ct=>{
            const msg = sprintf("%17s %40s %s",ct.hostname,ct.filename,ct.type)
            ns.tprint(msg);
        })

        return;
    }
    const flag = parse(ns.args.map(x=>x.toString()));
    if(flag.h || flag.help){
        ns.tprint("HELP : ")
        ns.tprint("run cmd [filename] [--host|-t]");
        return;
    }
    let hostFlag : string|undefined;
    if(flag.host || flag.t){
        hostFlag = flag.host.toString() || flag.t.toString();
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        if(!ns.serverExists(hostFlag!)){
            ns.tprint("unexist hostname");
            return;
        }
    }
    const inputFilename = flag._.toString();
    const p = searchFilename(ns,inputFilename, hostFlag);
    if(p === null){
        ns.tprint(`could not file ${inputFilename}`);
        return;
    }
    const {filename, hostname: target} = p;    
    
    ns.tprint(`detail of ${target}:${filename}`);

    const msg = [
        `${filename}(${capi.getNumTriesRemaining(filename,target)}/10)`,
        `${capi.getContractType(filename,target)}`,
        `${capi.getDescription(filename,target).replaceAll("&nbsp;"," ")}`,
        `${JSON.stringify(capi.getData(filename,target))}`
    ].join("\n");
    ns.tprint("\n",msg);
}

// eslint-disable-next-line @typescript-eslint/no-unused-vars
export function autocomplete(data : AutocompleteData, args : string[]) : string[] {
    if(args.length == 1){
        return [...data.servers]
    }
    return [];
}